// test.cpp: dfinit le point d'entre pour l'application console.
//


#include "stdafx.h"
#include <windows.h>
#include <assert.h>
#include <stdlib.h>
#include "MicroSecondTimer.h"

/** Some constants for the states machine */
namespace IR 
{
	const int   LISTENING_STATE    =  0;
	const int   LOW_PULSE_STATE    =  1;
	const int   HIGH_PULSE_STATE   =  2;
	const int   LAST_PULSE         =  3;
	const int   MAX_HIGH_PULSE     = 13;
	const short PIN_15_MASK        =  8;
	const __int64 MAX_WAITING_TIME =   4000; // Unit = microsecond
};

using namespace IR;

HANDLE hComm;
HANDLE g_hThread;
DWORD g_dwThreadId;
BOOL g_bListening;
OVERLAPPED g_ov;
OVERLAPPED o;

BOOL WriteABuffer(char * lpBuf, DWORD dwToWrite)
{
	OVERLAPPED osWrite = {0};
	DWORD dwWritten;
	DWORD dwRes;
	BOOL fRes;

	// Create this write operation's OVERLAPPED structure's hEvent.
	osWrite.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	if (osWrite.hEvent == NULL)
	{
		// error creating overlapped event handle
		return FALSE;
	}

	// Issue write.
	if (!WriteFile(hComm, lpBuf, dwToWrite, &dwWritten, &osWrite)) 
	{
		if (GetLastError() != ERROR_IO_PENDING) 
		{ 
			// WriteFile failed, but isn't delayed. Report error and abort.
			fRes = FALSE;
		}
		else
		{
			// Write is pending.
			dwRes = WaitForSingleObject(osWrite.hEvent, INFINITE);
			switch(dwRes)
			{
				// OVERLAPPED structure's event has been signaled. 
			case WAIT_OBJECT_0:
				if (!GetOverlappedResult(hComm, &osWrite, &dwWritten, FALSE))
					fRes = FALSE;
				else
					// Write operation completed successfully.
					fRes = TRUE;
				break;

			default:
				// An error has occurred in WaitForSingleObject.
				// This usually indicates a problem with the
				// OVERLAPPED structure's event handle.
				fRes = FALSE;
				break;
			} // End of switch(dwRes)
		}
	}
	else
	{
		// WriteFile completed immediately.
		fRes = TRUE;
	}

	CloseHandle(osWrite.hEvent);
	return fRes;
}

int OpenComPort(TCHAR *comPort, DWORD overlapped)
{
	// Par dfaut RTS est  +11 VDC
	hComm = CreateFile( /*gszPort*/ comPort, 
		GENERIC_READ | GENERIC_WRITE, 
		0, 
		0, 
		OPEN_EXISTING,
		overlapped,
		NULL);

	if (hComm == INVALID_HANDLE_VALUE)
	{
		MessageBox(NULL, L"Problme d'ouverture de COM1 ", L"Message", MB_OK);
	}

	DWORD dwStoredFlags;

	dwStoredFlags = EV_BREAK | EV_CTS    | EV_DSR    | EV_ERR     | EV_RING |\
		EV_RLSD | EV_RXCHAR | EV_RXFLAG | EV_TXEMPTY ;

	if (!SetCommMask(hComm, dwStoredFlags))
		MessageBox(NULL, L"Problme de configuration de COM1", L"Message" ,MB_OK);

	DCB dcbBackup = {0};
	DCB dcb = {0};

	if (!GetCommState(hComm, &dcbBackup))
	{
		MessageBox(NULL, L"Impossible d'obtenir la configuration de COM1", L"Message" ,MB_OK);
		return 1;	
	}

	memcpy(&dcb, &dcbBackup, sizeof(DCB));

	memset(&dcb, 0, sizeof(dcb));
	dcb.BaudRate = CBR_9600;
	dcb.ByteSize = 8;
	dcb.DCBlength = sizeof(DCB);

	SetCommState(hComm, &dcb);

	return 0;
}

DWORD WINAPI ThreadProc( LPVOID lpParam ) 
{ 
	DWORD dwCommEvent;
	DWORD dwModemState;
	TCHAR dbgstr[32];
	int state = LISTENING_STATE;

	memset(&dbgstr[0], 0, sizeof(dbgstr));

	for(;;)
	{
		if (!WaitCommEvent(hComm, &dwCommEvent, NULL))
		{
			// An error occurred waiting for the event.
			return FALSE;
		}
		else
		{
			// Event has occurred.
			GetCommModemStatus(hComm, &dwModemState);
			wsprintf(dbgstr, L"%08x", dwModemState);

			if( (dwModemState & MS_RLSD_ON) == MS_RLSD_ON)
			{
				wprintf(L"CD event %s\n", dbgstr);
			}
			else if(dwCommEvent & EV_DSR)
			{
				wprintf(L"DSR event %s\n", dbgstr);
			}
			else if(dwCommEvent & EV_BREAK)
			{
				wprintf(L"BREAK event %s\n", dbgstr);
				break;
			}
			else
			{
				wprintf(L"Unknown event %s\n", dbgstr);
			}
		}
	}

	return ::MessageBox(NULL, TEXT(""), TEXT(""), MB_OK); 
} 

void ReportStatusEvent(DWORD dwCommEvent)
{
	wprintf(TEXT("ReportStatusEvent : %08x\n"), dwCommEvent);
}

void DoBackgroundWork()
{
	wprintf(TEXT("DoBackgroundWork\n"));
}

// #define STATUS_CHECK_TIMEOUT      30   // 30 Milliseconds for RECS80
#define STATUS_CHECK_TIMEOUT      30   // 30 Milliseconds for RECS80

DWORD WINAPI ThreadProc2( LPVOID lpParam ) 
{
	DWORD      dwOvRes;
	DWORD      dwCommEvent;
	DWORD      dwStoredFlags;
	BOOL      fWaitingOnStat = FALSE;
	OVERLAPPED osStatus = {0};
	DWORD dwModemState;
	bool dsrEvent = false;
	short receivedValue = 0;
	CMicroSecondTimer *timer;
	__int64 Frq_Base, deltaTime, T0, T1;
	int lowPulsesNbr = 0;
	int state = 0;
	int firstDownUp = 0;

	try
	{
		timer = new CMicroSecondTimer();
	}
	catch(int exc)
	{
		MessageBox(NULL, L"Problem with precision timer", L"Aborting...", MB_ICONERROR);
		return 1;
	}

	osStatus.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	if (osStatus.hEvent == NULL)
	{
		// error creating event; abort
		return 0;
	}

	lowPulsesNbr = 0;
	receivedValue = 0;
	int index = 0;

	//   M a i n   l o o p
	for ( ; ; ) 
	{
		// Issue a status event check if one hasn't been issued already.
		if (!fWaitingOnStat) 
		{
			if (!WaitCommEvent(hComm, &dwCommEvent, &osStatus)) 
			{
				if (GetLastError() == ERROR_IO_PENDING)
				{
					fWaitingOnStat = TRUE;
				}
				else
				{
					// error in WaitCommEvent; abort
					break;
				}
			}
			else
			{
				// WaitCommEvent returned immediately.
				// Deal with status event as appropriate.
				ReportStatusEvent(dwCommEvent); 
			}
		}

		// Check on overlapped operation.
		if (fWaitingOnStat) 
		{
			// Wait a little while for an event to occur.
			dwOvRes = WaitForSingleObject(osStatus.hEvent, STATUS_CHECK_TIMEOUT);

			switch(dwOvRes)
			{
				// Event occurred.
			case WAIT_OBJECT_0:
				if (!GetOverlappedResult(hComm, &osStatus, &dwOvRes, FALSE))
				{
					// An error occurred in the overlapped operation;
					// call GetLastError to find out what it was
					// and abort if it is fatal.
				}
				else
					// Status event is stored in the event flag
					// specified in the original WaitCommEvent call.
					// Deal with the status event as appropriate.
					GetCommModemStatus(hComm, &dwModemState);
				if(dwCommEvent == EV_DSR)
				{
					// wprintf(L"EV_DSR : dwModemState = %08x\n", dwModemState);
					// wprintf(L"EV_DSR : %s\n", dwModemState == MS_DSR_ON ? TEXT("Up") : TEXT("Down"));
					dsrEvent = true;
					if(dwModemState == MS_DSR_ON)
					{
						// U p    p u l s e
						T0 = timer->SampleTimeAsMicroS();
						// printf("Duration = %I64d microseconds\n", T1-T0);
					}
					else
					{
						// D o w n    p u l s e
						T1 = timer->SampleTimeAsMicroS();
						if(firstDownUp == 0)
						{
							firstDownUp++;
						}
						else if(firstDownUp == 1)
						{
							deltaTime = T1 - T0;
							printf("Duration = %I64d \346s\n", deltaTime);
							if(abs(deltaTime - 2000) >= 2000) 
							{
								receivedValue |= (1 << index);
							}
							index++;
						}
						lowPulsesNbr++;
					}

				}

				// ReportStatusEvent(dwCommEvent);

				dwCommEvent = 0;
				// Set fWaitingOnStat flag to indicate that a new
				// WaitCommEvent is to be issued.
				fWaitingOnStat = FALSE;
				break;

			case WAIT_TIMEOUT:
				// Operation isn't complete yet. fWaitingOnStatusHandle flag 
				// isn't changed since I'll loop back around and I don't want
				// to issue another WaitCommEvent until the first one finishes.
				//
				// This is a good time to do some background work.
				if(dsrEvent && lowPulsesNbr == 13)
				{
					wprintf(L"Received = 0x%04x  Low pulses =%d\n", 
						receivedValue,
						lowPulsesNbr);
					dsrEvent = false;

				}

				lowPulsesNbr = 0;
				receivedValue = 0;
				index = 0;
				firstDownUp = 0;

				break;                       

			default:
				// Error in the WaitForSingleObject; abort
				// This indicates a problem with the OVERLAPPED structure's
				// event handle.
				CloseHandle(osStatus.hEvent);
				return 0;
			}
		}
	} //   E n d   o f   m a i n   l o o p

	CloseHandle(osStatus.hEvent);

	delete timer;
}

int _tmain(int argc, _TCHAR* argv[])
{
	OpenComPort(TEXT("COM1"), FILE_FLAG_OVERLAPPED);

	g_bListening = TRUE;

	g_hThread = CreateThread( 
		NULL,              // default security attributes
		0,                 // use default stack size  
		ThreadProc2,        // thread function 
		NULL,              // argument to thread function 
		0,                 // use default creation flags 
		&g_dwThreadId);    // returns the thread identifier 


	int l_currentChar = 0;
	char l_buffer[2];
	l_buffer[0] = 0x10;
	DWORD l_dwNumberBytesWritten;

	while((l_currentChar = fgetc(stdin)) != 'q')
	{
		if(l_currentChar == 's')
		{
			// 
			TerminateThread(g_hThread, 0);
			// Sleep(500);
		}

		if(l_currentChar == 'w')
		{
			SetCommBreak(hComm);
		}

		fflush(stdin);
	}

	TerminateThread(g_hThread, 0);



	DWORD      dwRes;

	BOOL       fWaitingOnStat = FALSE;
	OVERLAPPED osStatus = {0};
	DWORD	  dwOvRes;

	osStatus.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	if (osStatus.hEvent == NULL)
		// error creating event; abort
		return 0;

	/*
	for ( ; ; ) {
	// Issue a status event check if one hasn't been issued already.

	//      if (!fWaitingOnStat) 
	{
	if (!WaitCommEvent(hComm, &dwCommEvent, &osStatus)) {
	if (GetLastError() == ERROR_IO_PENDING)
	// bWaitingOnStatusHandle = TRUE;
	continue;
	else
	// error in WaitCommEvent; abort
	break;
	}
	else
	// WaitCommEvent returned immediately.
	// Deal with status event as appropriate.
	// ReportStatusEvent(dwCommEvent); 
	MessageBox(NULL, L"", L"", MB_OK);
	continue;
	}

	// Check on overlapped operation.
	// if (fWaitingOnStat) 
	{
	// Wait a little while for an event to occur.
	dwRes = WaitForSingleObject(osStatus.hEvent, STATUS_CHECK_TIMEOUT);
	switch(dwRes)
	{
	// Event occurred.
	case WAIT_OBJECT_0: 
	if (!GetOverlappedResult(hComm, &osStatus, &dwOvRes, FALSE))
	// An error occurred in the overlapped operation;
	// call GetLastError to find out what it was
	// and abort if it is fatal.
	continue;
	else
	// Status event is stored in the event flag
	// specified in the original WaitCommEvent call.
	// Deal with the status event as appropriate.
	// ReportStatusEvent(dwCommEvent);
	MessageBox(NULL, L"", L"", MB_OK);

	// Set fWaitingOnStat flag to indicate that a new
	// WaitCommEvent is to be issued.
	fWaitingOnStat = FALSE;
	break;

	case WAIT_TIMEOUT:
	// Operation isn't complete yet. fWaitingOnStatusHandle flag 
	// isn't changed since I'll loop back around and I don't want
	// to issue another WaitCommEvent until the first one finishes.
	//
	// This is a good time to do some background work.
	// DoBackgroundWork();
	break;                       

	default:
	// Error in the WaitForSingleObject; abort
	// This indicates a problem with the OVERLAPPED structure's
	// event handle.
	CloseHandle(osStatus.hEvent);
	return 0;
	}
	}
	}
	*/

	CloseHandle(osStatus.hEvent);

	// Restore DCB
	//	SetCommState(hComm, &dcbBackup);

	return 0;
}

